/**********************************************************************\
**
**                         Copyright (c) 2004-2017 by
**                          HENGSTLER GmbH, Aldingen
**
**      This  software  is furnished under a licence and may be used and
**      copied only in accordance with the terms  of  such  licence  and
**      with the inclusion of the above copyright notice.  This software
**      or any other copies thereof may not  be  provided  or  otherwise
**      made  available  to any other person.  No title to and ownership
**      of the software is hereby transferred.
**
**      The  information  in  this software is subject to change without
**      notice and should not be construed as a commitment by Hengstler.
**
**  Module:      C56SdkDemo06DlgMain.cpp
**  Facility:    S684002 - Hengstler C56 Printer Driver for Windows XP/7/8/10
**  Component:   C56 SDK Demo Application 06 (C56SdkDemo06)
**  Date         2017APR27
**  Environment: Windows XP/7/8/10
**  Compiler:    Microsoft Visual Studio 2015
**  Abstract:    Implementation of main-dialog of SDK C-56 Demo 06
**  Notes:       -
**
\**********************************************************************/

/*
** INCLUDES
*/

#include "stdafx.h"

#include <process.h> 

#include "..\..\Libraries\C56SDK.h"  /* SDK */

#include "C56SdkDemo06.h"
#include "C56SdkDemo06DlgMain.h"

/*
** COMPILER
*/

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

/*
** MESSAGE MAP
*/

BEGIN_MESSAGE_MAP(CC56SdkDemo06DlgMain, CDialog)
   ON_WM_SYSCOMMAND()
   ON_WM_PAINT()
   ON_WM_QUERYDRAGICON()
	//}}AFX_MSG_MAP
   ON_BN_CLICKED(IDC_BUTTON_MODIFY, OnBnClickedModify)
   ON_BN_CLICKED(IDC_BUTTON_SAVE,   OnBnClickedSave)
   ON_BN_CLICKED(IDC_BUTTON_LOAD,   OnBnClickedLoad)
END_MESSAGE_MAP()

//******************************************************************************************
// METHOD:   CC56SdkDemo06DlgMain::CC56SdkDemo06DlgMain
// ABSTRACT: Object constructor
//******************************************************************************************

CC56SdkDemo06DlgMain::CC56SdkDemo06DlgMain
   (CWnd* pParent /*=NULL*/)
:  CDialog(CC56SdkDemo06DlgMain::IDD, pParent)
{
   //-- Set member icon
   m_hIcon = AfxGetApp()->LoadIcon(IDR_C56);
}

//******************************************************************************************
// METHOD:   CC56SdkDemo06DlgMain::DoDataExchange
// ABSTRACT: Called by the framework to exchange and validate dialog data
//******************************************************************************************

void CC56SdkDemo06DlgMain::DoDataExchange(CDataExchange* pDX)
{
   CDialog::DoDataExchange(pDX);
}

//******************************************************************************************
// METHOD:   CC56SdkDemo06DlgMain::OnInitDialog
// ABSTRACT: This member function is called in response to the WM_INITDIALOG message.
//******************************************************************************************

BOOL CC56SdkDemo06DlgMain::OnInitDialog
   ()
{
   //-- Initialize dialog
   CDialog::OnInitDialog();

   //-- Check IDM_ABOUTBOX for being in system menu
   ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
   ASSERT(IDM_ABOUTBOX < 0xF000);

   //-- Invalidate file name
   strcpy_s(m_szFile, sizeof(m_szFile), ""); 

   //-- Set C-56 icon for dialog 
   SetIcon(m_hIcon, TRUE);   // Use large symbol verwenden
   SetIcon(m_hIcon, FALSE);  // Use small symbol 

   //-- Setup printer selection combo box with available C-56 printers
   { 
      HANDLE hEnumPrinter; // Handle for enumeration 

      //-- Start access SDK to go through all C-56 printers 
      if ((hEnumPrinter = C56Sdk_PmStartEnumPrinter()) != NULL) {

         LPSTR       lpszPrinter; // Name of printer
         CComboBox * pPrinterList;  // Pointer to combo box item

         //-- Get printer selection combo box item
         if (pPrinterList = ((CComboBox *) this->GetDlgItem(IDC_COMBO_PRINTER_LIST))) {

            //-- Clear printer selection combo box
            pPrinterList->ResetContent();

            //-- Go through all C-56 printers
            for ( lpszPrinter = C56Sdk_PmEnumPrinter(hEnumPrinter, TRUE)
                ; lpszPrinter != NULL
                ; lpszPrinter = C56Sdk_PmEnumPrinter(hEnumPrinter, FALSE)
                ) {

               //-- Add printer to printer selection combo box 
               pPrinterList->AddString(lpszPrinter);
            }

            //-- Select 1st printer in printer selection combo box
            pPrinterList->SetCurSel(0);
         }
         
         //-- End of enumeration, free handle
         C56Sdk_PmEndEnumPrinter(hEnumPrinter);
      } 
   }
      
   //-- Initialization successfull, set focus to first item
   return TRUE;  
}

//******************************************************************************************
// METHOD:   CC56SdkDemo06DlgMain::OnSysCommand
// ABSTRACT: The framework calls this member function when the user selects a command from 
//           the Control menu, or when the user selects the Maximize or the Minimize button.
//******************************************************************************************

void CC56SdkDemo06DlgMain::OnSysCommand
   (UINT nID, LPARAM lParam)
{
   //-- Call original CDialog::OnSysCommand member function
   CDialog::OnSysCommand(nID, lParam);
}

//******************************************************************************************
// METHOD:   CC56SdkDemo06DlgMain::OnPaint
// ABSTRACT: The framework calls this member function when Windows or an application makes a 
//           request to repaint a portion of an application's window.
//******************************************************************************************

void CC56SdkDemo06DlgMain::OnPaint
   () 
{
   //-- Check if dialog is iconic
   if (IsIconic()) {

      CPaintDC dc(this); // Device context for paint 

      //-- Erase background of icon
      SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

      //-- Center symbol in clientr rectangle
      int cxIcon = GetSystemMetrics(SM_CXICON);
      int cyIcon = GetSystemMetrics(SM_CYICON);
      CRect rect;
      GetClientRect(&rect);
      int x = (rect.Width() - cxIcon + 1) / 2;
      int y = (rect.Height() - cyIcon + 1) / 2;

      //-- Draw symbol 
      dc.DrawIcon(x, y, m_hIcon);
   }
   else {

      //-- Call original CDialog::OnPaint member function
      CDialog::OnPaint();
   }
}

//******************************************************************************************
// METHOD:   CC56SdkDemo06DlgMain::OnQueryDragIcon
// ABSTRACT: The framework calls this member function by a minimized (iconic) window that 
//           does not have an icon defined for its class.
//******************************************************************************************

HCURSOR CC56SdkDemo06DlgMain::OnQueryDragIcon
   ()
{
   return static_cast<HCURSOR>(m_hIcon);
}

//******************************************************************************************
// METHOD:   CC56SdkDemo06DlgMain::OnBnClickedModify
// ABSTRACT: Calls control.exe to open printers&faxes dialog
//******************************************************************************************

void CC56SdkDemo06DlgMain::OnBnClickedModify
   ()
{
   //-- Start printers and faxes dialog
   WinExec("control.exe printers", 0);
}

//******************************************************************************************
// METHOD:   CC56SdkDemo06DlgMain::OpenFileName
// ABSTRACT: Handles file selection dialog for save/open file
//******************************************************************************************

BOOL CC56SdkDemo06DlgMain::OpenFileName
   (HWND hWnd, PSTR pszFile, INT nFile, PSTR pszTile, PSTR pszFilter, BOOL fSave)
{
   BOOL fResult = FALSE;                  // Result of this function
   OPENFILENAME  sOpenFileName;           // OpenFileName structure
   
   //-- Initialize OPENFILENAME structure
   ZeroMemory(&sOpenFileName, sizeof(sOpenFileName));
   sOpenFileName.lStructSize     = sizeof(sOpenFileName);
   sOpenFileName.hwndOwner       = hWnd;
   sOpenFileName.lpstrTitle      = pszTile;
   sOpenFileName.lpstrFile       = pszFile;
   sOpenFileName.nMaxFile        = nFile;
   sOpenFileName.lpstrFilter     = pszFilter;
   sOpenFileName.nFilterIndex    = 1;
   sOpenFileName.lpstrFileTitle  = NULL;
   sOpenFileName.nMaxFileTitle   = 0;
   sOpenFileName.lpstrInitialDir = NULL;
   sOpenFileName.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST;

   //-- Display the open/saev file name dialog box. 
   fResult = (fSave ? GetSaveFileNameA(&sOpenFileName) : GetOpenFileNameA(&sOpenFileName));

   //-- Return result 
   return(fResult);
}

//******************************************************************************************
// METHOD:   CC56SdkDemo06DlgMain::OnBnClickedSave
// ABSTRACT: Saves printer properties to a file
//******************************************************************************************

void CC56SdkDemo06DlgMain::OnBnClickedSave
   ()
{
   CComboBox  * pPrinterList;               // Pointer to combo box item
   PSTR         pszProperties = NULL;       // Storage for printer properties
   INT          iSize;                      // Size of storage

   //-- Get name of save file 
   if (this->OpenFileName(this->GetSafeHwnd(), m_szFile, sizeof(m_szFile), "SELECT C-56 PRINTER PROPERTIES FILE TO SAVE INTO ...", "C-56 PRINTER PROPERTIES (*.C56PP)\0*.C56PP\0All(*.*)\0*.*\0", TRUE)) {

      //-- Split this path and add extension if not already done
      {
         char szDrive[_MAX_DRIVE];
         char szDir[_MAX_DIR];
         char szFname[_MAX_FNAME];
         char szExt[_MAX_EXT];

         //-- Split path and set extension to C56PP if no extension set
         _splitpath_s(m_szFile, szDrive, sizeof(szDrive), szDir, sizeof(szDir), szFname, sizeof(szFname), szExt, sizeof(szExt));
         if (!strlen(szExt)) strcpy_s(szExt, sizeof(szExt), "C56PP");
         _makepath_s(m_szFile, sizeof(m_szFile), szDrive, szDir, szFname, szExt);
      }
      
      //-- Get printer list (dialog item combo box)
      if ((pPrinterList = ((CComboBox *) this->GetDlgItem(IDC_COMBO_PRINTER_LIST))) != NULL) {

         INT iPrinterSel; // Selected printer
 
         //-- Determine selected printer
         if ((iPrinterSel = pPrinterList->GetCurSel()) != CB_ERR) {
 
            LPSTR lpszPrinter = NULL;  // Printer name

            //-- Allocate memory for this printer and get name from combo box
            if (    ((lpszPrinter = (LPSTR) malloc(pPrinterList->GetLBTextLen(iPrinterSel)+1)) != NULL) 
                 && (pPrinterList->GetLBText(iPrinterSel, lpszPrinter) != CB_ERR)
               ) {

               //-- Determine size of printer properties
               if ((iSize = C56Sdk_PmGetProperties(lpszPrinter, NULL, 0)) > 0) {

                  //-- Allocate memory for printer properties
                  if ((pszProperties = (PSTR) malloc(iSize)) != NULL) {

                     //-- Get printer properties
                     if (C56Sdk_PmGetProperties(lpszPrinter, pszProperties, iSize) > 0) {

                        CFile cFile; 

                        //-- Open file, create if required and truncate if available
                        if (cFile.Open( m_szFile, CFile::modeCreate | CFile::modeWrite, NULL)) {

                            //-- Write cto the file
                            cFile.Write(pszProperties, iSize);

                            //-- Close the file
                            cFile.Close();
                        }
                     }

                     //-- Free allocated data 
                     free(pszProperties); pszProperties = NULL;
                  }
               }

               //-- Free allocated memory
               if (lpszPrinter) free(lpszPrinter); lpszPrinter = NULL; 
            }
         }
      }
   }

   //-- Free allocated data 
   if (pszProperties) free(pszProperties); pszProperties = NULL;
}

//******************************************************************************************
// METHOD:   CC56SdkDemo06DlgMain::OnBnClickedLoad
// ABSTRACT: Loads printer properties from a file
//******************************************************************************************

void CC56SdkDemo06DlgMain::OnBnClickedLoad
   ()
{
   CComboBox  * pPrinterList;               // Pointer to combo box item
   PSTR         pszProperties = NULL;       // Storage for printer properties
   INT          iSize;                      // Size of storage

   //-- Get name of save file 
   if (this->OpenFileName(this->GetSafeHwnd(), m_szFile, sizeof(m_szFile), "SELECT C-56 PRINTER PROPERTIES FILE TO LOAD FROM ...", "C-56 PRINTER PROPERTIES (*.C56PP)\0*.C56PP\0All(*.*)\0*.*\0", FALSE)) {

      //-- Get printer list (dialog item combo box)
      if ((pPrinterList = ((CComboBox *) this->GetDlgItem(IDC_COMBO_PRINTER_LIST))) != NULL) {

         INT iPrinterSel; // Selected printer
 
         //-- Determine selected printer
         if ((iPrinterSel = pPrinterList->GetCurSel()) != CB_ERR) {
 
            LPSTR lpszPrinter = NULL;  // Printer name

            //-- Allocate memory for this printer and get name from combo box
            if (    ((lpszPrinter = (LPSTR) malloc(pPrinterList->GetLBTextLen(iPrinterSel)+1)) != NULL) 
                 && (pPrinterList->GetLBText(iPrinterSel, lpszPrinter) != CB_ERR)
               ) {

               //-- Determine size of printer properties
               if ((iSize = C56Sdk_PmGetProperties(lpszPrinter, NULL, 0)) > 0) {

                  //-- Allocate memory for printer properties
                  if ((pszProperties = (PSTR) malloc(iSize)) != NULL) {

                     CFile cFile; 

                     //-- Open file, create if required and truncate if available
                     if (cFile.Open( m_szFile, CFile::modeRead | CFile::shareDenyWrite, NULL)) {

                        //-- Check file length being length property 
                        if (cFile.GetLength() == iSize) {

                           //-- Read printer properties from the file
                           if (cFile.Read(pszProperties, iSize) == iSize) {

                              //-- Set printer properties
                              C56Sdk_PmSetProperties(lpszPrinter, pszProperties, iSize);
                           }
                        }

                        //-- Close the file
                        cFile.Close();
                     }

                     //-- Free allocated data 
                     free(pszProperties); pszProperties = NULL;
                  }
               }

               //-- Free allocated memory
               if (lpszPrinter) free(lpszPrinter); lpszPrinter = NULL; 
            }
         }
      }
   }

   //-- Free allocated data 
   if (pszProperties) free(pszProperties); pszProperties = NULL;
}

