/**********************************************************************\
**
**                         Copyright (c) 2006-2017 by
**                          HENGSTLER GmbH, Aldingen
**
**      This  software  is furnished under a licence and may be used and
**      copied only in accordance with the terms  of  such  licence  and
**      with the inclusion of the above copyright notice.  This software
**      or any other copies thereof may not  be  provided  or  otherwise
**      made  available  to any other person.  No title to and ownership
**      of the software is hereby transferred.
**
**      The  information  in  this software is subject to change without
**      notice and should not be construed as a commitment by Hengstler.
**
**  Module:      C56SdkDemo10DlgMain.cpp
**  Facility:    S684002 - Hengstler C56 Printer Driver for Windows XP/7/8/10
**  Component:   C56 SDK Demo Application 10 (C56SdkDemo10)
**  Date         2017APR27
**  Environment: Windows XP/7/8/10
**  Compiler:    Microsoft Visual Studio 2015
**  Abstract:    Implementation of main-dialog of SDK C-56 Demo 10
**  Notes:       -
**
\**********************************************************************/

/*
** INCLUDES
*/

#include "stdafx.h"

#include <winspool.h>

#include "..\..\Libraries\C56SDK.h"  /* SDK */

#include "C56SdkDemo10.h"
#include "C56SdkDemo10DlgMain.h"

/*
** COMPILER
*/

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

/*
** MACROS
*/ 

//-- Macros for writing/reading data to through printer I/O
#define C56SDK_PIOPRINTERWRITE_STRING(hHandle,pszStr,dwTimeOutMs)     C56Sdk_PioPrinterWrite(hHandle, (unsigned char *)pszStr, (DWORD) strlen(pszStr), dwTimeOutMs)
#define C56SDK_PIOPRINTERWRITE_BYTEARRAY(hHandle, aubDat,dwTimeOutMs)  C56Sdk_PioPrinterWrite(hHandle, (unsigned char *)aubDat, (DWORD) sizeof(aubDat), dwTimeOutMs)
#define C56SDK_PIOPRINTERREAD_BYTEARRAY(hHandle, aubDat,dwTimeOutMs)   C56Sdk_PioPrinterRead(hHandle,  (unsigned char *)aubDat, (DWORD) sizeof(aubDat), dwTimeOutMs)

/*
** MESSAGE MAP
*/

BEGIN_MESSAGE_MAP(CC56SdkDemo10DlgMain, CDialog)
   ON_WM_SYSCOMMAND()
   ON_WM_PAINT()
   ON_WM_QUERYDRAGICON()
   ON_WM_TIMER()
   //}}AFX_MSG_MAP
   ON_BN_CLICKED(IDC_BUTTON_SINGLE_PRINTOUT, OnBnClickedButtonSinglePrintout)
   ON_BN_CLICKED(IDC_BUTTON_AUTO_PRINTOUT,   OnBnClickedButtonAutoPrintout)

END_MESSAGE_MAP()

//******************************************************************************************
// METHOD:   CC56SdkDemo10DlgMain::CC56SdkDemo10DlgMain
// ABSTRACT: Object constructor
//******************************************************************************************

CC56SdkDemo10DlgMain::CC56SdkDemo10DlgMain
   (CWnd* pParent /*=NULL*/)
:  CDialog(CC56SdkDemo10DlgMain::IDD, pParent)
{
   //-- Set member icon
   m_hIcon = AfxGetApp()->LoadIcon(IDR_C56);
}

//******************************************************************************************
// METHOD:   CC56SdkDemo10DlgMain::DoDataExchange
// ABSTRACT: Called by the framework to exchange and validate dialog data
//******************************************************************************************

void CC56SdkDemo10DlgMain::DoDataExchange(CDataExchange* pDX)
{
   CDialog::DoDataExchange(pDX);
}

//******************************************************************************************
// METHOD:   CC56SdkDemo10DlgMain::OnInitDialog
// ABSTRACT: This member function is called in response to the WM_INITDIALOG message.
//******************************************************************************************

BOOL CC56SdkDemo10DlgMain::OnInitDialog
   ()
{
   //-- Initialize dialog
   CDialog::OnInitDialog();

   //-- Create a timer for update of status
   m_puiTimerUpdateStatus = this->SetTimer(1, 500, 0);

   //-- Invalidate job number 
   m_bJobNr = 0xFF;
 
    //-- Initialize member variables 
   {
      this->m_strPrinter  = "";
      this->m_dwTicketNr        = 1;
      this->m_dwNrOfTickets     = 10000;
      this->m_dwIntervalMs      = 2500;
      this->m_fPrinting         = FALSE;
      this->m_strStatus = "\nPRINTING DISABLED";
      this->m_strError  = "-";

      //-- Load these dialog items
      this->LoadDlgItems();
   }

   //-- Set C-56 icon for dialog 
   SetIcon(m_hIcon, TRUE);   // Use large symbol verwenden
   SetIcon(m_hIcon, FALSE);  // Use small symbol 

   //-- Setup printer selection combo box with available C-56 printers
   { 
      HANDLE hEnumPrinter; // Handle for enumeration 

      //-- Start access SDK to go through all C-56 printers 
      if ((hEnumPrinter = C56Sdk_PmStartEnumPrinter()) != NULL) {

         LPSTR       lpszPrinter; // Name of printer
         CComboBox * pPrinterList;  // Pointer to combo box item

         //-- Get printer selection combo box item
         if (pPrinterList = ((CComboBox *) this->GetDlgItem(IDC_COMBO_PRINTER_LIST))) {

            //-- Clear printer selection combo box
            pPrinterList->ResetContent();

            //-- Go through all C-56 printers
            for ( lpszPrinter = C56Sdk_PmEnumPrinter(hEnumPrinter, TRUE)
                ; lpszPrinter != NULL
                ; lpszPrinter = C56Sdk_PmEnumPrinter(hEnumPrinter, FALSE)
                ) {

               //-- Add printer to printer selection combo box 
               pPrinterList->AddString(lpszPrinter);
            }

            //-- Select 1st printer in printer selection combo box
            pPrinterList->SetCurSel(0);
         }
         
         //-- End of enumeration, free handle
         C56Sdk_PmEndEnumPrinter(hEnumPrinter);
      } 
   }
      

  //-- Set dialog items
   this->SetDlgItems();

   //-- Initialization successfull, set focus to first item
   return TRUE;  
}

/******************************************************************************************\
** CLASS  CC56SdkDemo10DlgMain
** METHOD LoadDlgItems
\******************************************************************************************/

void CC56SdkDemo10DlgMain::LoadDlgItems
   ()
{
   //-- Get dialog item settings from ini file
   {
      this->m_strPrinter = theApp.GetProfileString("CC56SdkDemo10DlgMain", "PrinterDriver",   "");
      this->m_dwTicketNr       = theApp.GetProfileInt   ("CC56SdkDemo10DlgMain", "TicketNr",         1);
      this->m_dwNrOfTickets    = theApp.GetProfileInt   ("CC56SdkDemo10DlgMain", "NrOfTickets",   1000);
      this->m_dwIntervalMs     = theApp.GetProfileInt   ("CC56SdkDemo10DlgMain", "IntervalMs",    2000);
   }
}

/******************************************************************************************\
** CLASS  CC56SdkDemo10DlgMain
** METHOD StoreDlgItems
\******************************************************************************************/

void CC56SdkDemo10DlgMain::StoreDlgItems
   ()
{
   //-- Write dialog item settings in ini file
   {
      theApp.WriteProfileString("CC56SdkDemo10DlgMain", "PrinterDriver", this->m_strPrinter);
      theApp.WriteProfileInt   ("CC56SdkDemo10DlgMain", "TicketNr",      this->m_dwTicketNr);
      theApp.WriteProfileInt   ("CC56SdkDemo10DlgMain", "NrOfTickets",   this->m_dwNrOfTickets);
      theApp.WriteProfileInt   ("CC56SdkDemo10DlgMain", "IntervalMs",    this->m_dwIntervalMs);
   }
}

/******************************************************************************************\
** CLASS  CC56SdkDemo10DlgMain
** METHOD SetDlgItems
\******************************************************************************************/

void CC56SdkDemo10DlgMain::SetDlgItems
   ()
{
   //-- Set dialog item: printer driver 
   {
      CComboBox       * pPrinterList = NULL;      // Pointer to combo box item
      
      //-- Get printer selection combo box item
      if (pPrinterList = ((CComboBox *) this->GetDlgItem(IDC_COMBO_PRINTER_LIST))) {
      
         int iSel; // Selected item

         //-- Select driver in combo box
         if ((iSel = pPrinterList->FindStringExact(0, this->m_strPrinter)) != CB_ERR)
            pPrinterList->SetCurSel(iSel);

         //-- Enable/disable window
         pPrinterList->EnableWindow(!this->m_fPrinting);
      }
   }

   //-- Setup dialog item: ticket number
   {
      CString strDlgItem;        // String containing dialog item data
      CEdit * pDlgItemEdit;      // Pointer to edit dialog item

      //-- Get edit field 
      if ((pDlgItemEdit = ((CEdit *) this->GetDlgItem(IDC_EDIT_TICKET_NR))) != NULL) {

         //-- Setup string with field
         strDlgItem.Format(_T("%d"), m_dwTicketNr);

         //-- Set edit field 
         pDlgItemEdit->SetWindowText(strDlgItem);

         //-- Enable/disable window
         pDlgItemEdit->EnableWindow(!this->m_fPrinting);
      }
   }

   //-- Setup dialog item: number of tickets to print
   {
      CString strDlgItem;        // String containing dialog item data
      CEdit * pDlgItemEdit;      // Pointer to edit dialog item

      //-- Get edit field 
      if ((pDlgItemEdit = ((CEdit *) this->GetDlgItem(IDC_EDIT_NR_OF_TICKETS))) != NULL) {

         //-- Setup string with field
         strDlgItem.Format(_T("%d"), m_dwNrOfTickets);

         //-- Set edit field 
         pDlgItemEdit->SetWindowText(strDlgItem);

         //-- Enable/disable window
         pDlgItemEdit->EnableWindow(!this->m_fPrinting);
      }
   }

   //-- Setup dialog item: print interval (MS)
   {
      CString strDlgItem;        // String containing dialog item data
      CEdit * pDlgItemEdit;      // Pointer to edit dialog item

      //-- Get edit field 
      if ((pDlgItemEdit = ((CEdit *) this->GetDlgItem(IDC_EDIT_INTERVAL_MS))) != NULL) {

         //-- Setup string with field
         strDlgItem.Format(_T("%d"), m_dwIntervalMs);

         //-- Set edit field 
         pDlgItemEdit->SetWindowText(strDlgItem);

         //-- Enable/disable window
         pDlgItemEdit->EnableWindow(!this->m_fPrinting);
      }
   }

   //-- Setup dialog item: print status
   {
      CString   strDlgItem;        // String containing dialog item data
      CStatic * pDlgItemText;      // Pointer to Text dialog item

      //-- Get edit field 
      if ((pDlgItemText = ((CStatic *) this->GetDlgItem(IDC_STATIC_STATUS))) != NULL) {

         //-- Set edit field 
         pDlgItemText->SetWindowText(this->m_strStatus);

         //-- Enable/disable window
         pDlgItemText->EnableWindow(this->m_fPrinting);
      }
   }

   //-- Setup dialog item: print error
   {
      CString   strDlgItem;        // String containing dialog item data
      CStatic * pDlgItemText;      // Pointer to Text dialog item

      //-- Get edit field 
      if ((pDlgItemText = ((CStatic *) this->GetDlgItem(IDC_STATIC_ERROR))) != NULL) {

         //-- Set edit field 
         pDlgItemText->SetWindowText(this->m_strError);

         //-- Enable/disable window
         pDlgItemText->EnableWindow(this->m_fPrinting);
      }
   }

   //-- Setup button to start or stop automatic printing
   {
      CButton * pDlgItemButton;      // Pointer to button dialog item

      //-- Get button field 
      if ((pDlgItemButton = ((CButton *) this->GetDlgItem(IDC_BUTTON_AUTO_PRINTOUT))) != NULL) {

          //-- Set edit field 
          pDlgItemButton->SetWindowText(this->m_fPrinting ? "Stop &Automatic Ticket Printout" : "Start &Automatic Ticket Printout");
      }
   }

   //-- Setup button for single printing
   {
      CButton * pDlgItemButton;      // Pointer to button dialog item

      //-- Get button field 
      if ((pDlgItemButton = ((CButton *) this->GetDlgItem(IDC_BUTTON_SINGLE_PRINTOUT))) != NULL) {

          //-- Enable/disable window
          pDlgItemButton->EnableWindow(!this->m_fPrinting);
      }
   }
}

/******************************************************************************************\
** CLASS  CC56SdkDemo10DlgMain
** METHOD GetDlgItems
\******************************************************************************************/

void CC56SdkDemo10DlgMain::GetDlgItems
   ()
{
   //-- Get dialog item: printer driver 
   {
      CComboBox       * pPrinterList = NULL;      // Pointer to combo box item
      
      this->m_strPrinter = "";

      //-- Get printer selection combo box item
      if (pPrinterList = ((CComboBox *) this->GetDlgItem(IDC_COMBO_PRINTER_LIST))) {

         INT nIndex;

         //-- Get actual selected driver 
         if ((nIndex = pPrinterList->GetCurSel()) != LB_ERR)
            pPrinterList->GetLBText(nIndex, this->m_strPrinter);
      }
   }

   //-- Get dialog item: ticket number
   {
      CString strDlgItem;        // String containing dialog item data
      CEdit * pDlgItemEdit;      // Pointer to edit dialog item

      //-- Get edit field 
      if ((pDlgItemEdit = ((CEdit *) this->GetDlgItem(IDC_EDIT_TICKET_NR))) != NULL) {

          //-- Get edit field contents
          pDlgItemEdit->GetWindowText(strDlgItem);

          //-- Set ticket number 
          m_dwTicketNr = abs(atoi(strDlgItem.GetBuffer())); 
          strDlgItem.ReleaseBuffer();
      }
   }

   //-- Get dialog item: number of tickets to print
   {
      CString strDlgItem;        // String containing dialog item data
      CEdit * pDlgItemEdit;      // Pointer to edit dialog item

      //-- Get edit field 
      if ((pDlgItemEdit = ((CEdit *) this->GetDlgItem(IDC_EDIT_NR_OF_TICKETS))) != NULL) {

          //-- Get edit field contents
          pDlgItemEdit->GetWindowText(strDlgItem);

          //-- Set number of tickets to print
          m_dwNrOfTickets = abs(atoi(strDlgItem.GetBuffer())); 
          strDlgItem.ReleaseBuffer();
      }
   }

   //-- Get dialog item: print interval (MS)
   {
      CString strDlgItem;        // String containing dialog item data
      CEdit * pDlgItemEdit;      // Pointer to edit dialog item

      //-- Get edit field 
      if ((pDlgItemEdit = ((CEdit *) this->GetDlgItem(IDC_EDIT_INTERVAL_MS))) != NULL) {

          //-- Get edit field contents
          pDlgItemEdit->GetWindowText(strDlgItem);

          //-- Set print interval 
          m_dwIntervalMs = abs(atoi(strDlgItem.GetBuffer())); 
          strDlgItem.ReleaseBuffer();
      }
   }
}

//******************************************************************************************
// METHOD:   CC56SdkDemo10DlgMain::OnSysCommand
// ABSTRACT: The framework calls this member function when the user selects a command from 
//           the Control menu, or when the user selects the Maximize or the Minimize button.
//******************************************************************************************

void CC56SdkDemo10DlgMain::OnSysCommand
   (UINT nID, LPARAM lParam)
{
    //-- Store dialog items on close 
   if (nID == SC_CLOSE) {

      //-- Get dialog item settings 
      this->GetDlgItems();

      //-- Store these dialog items
      this->StoreDlgItems();
   }

   //-- Call original CDialog::OnSysCommand member function
   CDialog::OnSysCommand(nID, lParam);
}

//******************************************************************************************
// METHOD:   CC56SdkDemo10DlgMain::OnPaint
// ABSTRACT: The framework calls this member function when Windows or an application makes a 
//           request to repaint a portion of an application's window.
//******************************************************************************************

void CC56SdkDemo10DlgMain::OnPaint
   () 
{
   //-- Check if dialog is iconic
   if (IsIconic()) {

      CPaintDC dc(this); // Device context for paint 

      //-- Erase background of icon
      SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

      //-- Center symbol in clientr rectangle
      int cxIcon = GetSystemMetrics(SM_CXICON);
      int cyIcon = GetSystemMetrics(SM_CYICON);
      CRect rect;
      GetClientRect(&rect);
      int x = (rect.Width() - cxIcon + 1) / 2;
      int y = (rect.Height() - cyIcon + 1) / 2;

      //-- Draw symbol 
      dc.DrawIcon(x, y, m_hIcon);
   }
   else {

      //-- Call original CDialog::OnPaint member function
      CDialog::OnPaint();
   }
}

//******************************************************************************************
// METHOD:   CC56SdkDemo10DlgMain::OnQueryDragIcon
// ABSTRACT: The framework calls this member function by a minimized (iconic) window that 
//           does not have an icon defined for its class.
//******************************************************************************************

HCURSOR CC56SdkDemo10DlgMain::OnQueryDragIcon
   ()
{
   return static_cast<HCURSOR>(m_hIcon);
}

/******************************************************************************************\
** CLASS  CC56SdkDemo10DlgMain
** METHOD OnBnClickedButtonAutoPrintout
\******************************************************************************************/

void CC56SdkDemo10DlgMain::OnBnClickedButtonAutoPrintout
   ()
{
   //-- Toggle printing flag
   this->m_fPrinting = !this->m_fPrinting;

   //-- Get dialog item settings 
   this->GetDlgItems();

   //-- Correct data 
   m_dwIntervalMs = max(m_dwIntervalMs, 1);

   //-- Store these dialog items
   this->StoreDlgItems();

   //-- Update printing status / error
   this->m_strStatus = (this->m_fPrinting ? "\nPRINTING ENABLED" : "\nPRINTING DISABLED");  
   this->m_strError  = "-";  

   //-- Update dialog items
   this->SetDlgItems();

   //-- Kill timer and create new one if required
   this->KillTimer(2); this->m_puiTimerPrintout = 0;
   if (this->m_fPrinting)
      this->m_puiTimerPrintout = this->SetTimer(2, m_dwIntervalMs, NULL);

   //-- Set focus on start stop button
   this->GetDlgItem(IDC_BUTTON_AUTO_PRINTOUT)->SetFocus();
}

/******************************************************************************************\
** CLASS  CC56SdkDemo10DlgMain
** METHOD OnBnClickedButtonAutoPrintout
\******************************************************************************************/

void CC56SdkDemo10DlgMain::OnBnClickedButtonSinglePrintout
   ()
{
   //-- Get dialog item settings 
   this->GetDlgItems();

   //-- Store these dialog items
   this->StoreDlgItems();

   //-- Update printing status / error
   this->m_strStatus = "-";  
   this->m_strError  = "-";  

   //-- Do printout 
   this->DoPrintout(FALSE);

   //-- Update printing status / error
   this->m_strStatus = "\nPRINTING DISABLED";  
   // this->m_strError  = "-";  

   //-- Update dialog items
   this->SetDlgItems();
}

//******************************************************************************************
// METHOD:   CC56SdkDemo10DlgMain::DoUpdateStatus
// ABSTRACT: This member function is to update status information displayed 
//******************************************************************************************

VOID CC56SdkDemo10DlgMain::DoUpdateStatus
   (VOID)
{
 CComboBox * pPrinterList;  // Pointer to combo box item

   //-- Get printer list (dialog item combo box)
   if ((pPrinterList = ((CComboBox *) this->GetDlgItem(IDC_COMBO_PRINTER_LIST))) != NULL) {

      INT iPrinterSel; // Selected printer

      //-- Determine selected printer
      if ((iPrinterSel = pPrinterList->GetCurSel()) != CB_ERR) {

         LPSTR lpszPrinter = NULL;  // Printer name

         //-- Allocate memory for this printer and get name from combo box
         if (    ((lpszPrinter = (LPSTR) malloc(pPrinterList->GetLBTextLen(iPrinterSel)+1)) != NULL) 
              && (pPrinterList->GetLBText(iPrinterSel, lpszPrinter) != CB_ERR)
            ) {

            unsigned char aubC56CmdGetPrinterStatus[] = {0x1D, 0x61, 0x01};   // command to "get printer status"
            unsigned char aubC56DatPrinterStatus[4];                          // C56 status DWORD as respond to command "get printer status"
            HANDLE        hPrinterIo;                                         // Handle for printer I/O

            //-- Open connection to printer
            if ((hPrinterIo = C56Sdk_PioPrinterOpen((LPSTR)(LPCSTR) this->m_strPrinter,100)) != NULL) {
            
               //--Write commnad "get printer status" to printer
               if (C56SDK_PIOPRINTERWRITE_BYTEARRAY(hPrinterIo, aubC56CmdGetPrinterStatus, 100)) {
               
                  //-- Read "printer status DWORD" from printer 
                  if (C56SDK_PIOPRINTERREAD_BYTEARRAY(hPrinterIo, aubC56DatPrinterStatus, 1000)) {
                  
                     //--Validate and update data in dialog
 
                     //-- Update printer status
                     {  
                        //-- Update dialog items
                        this->GetDlgItem(IDC_RADIO_STATUS_TEMP_ALERT)->EnableWindow(TRUE);
                        this->CheckDlgButton(IDC_RADIO_STATUS_TEMP_ALERT, (aubC56DatPrinterStatus[0] & C56SR_PRINTER_TEMPALERT)    != 0);
                        this->GetDlgItem(IDC_RADIO_STATUS_LAST_PRINTOUT_LOST)->EnableWindow(TRUE);
                        this->CheckDlgButton(IDC_RADIO_STATUS_LAST_PRINTOUT_LOST, (aubC56DatPrinterStatus[0] & C56SR_PRINTER_PRINTOUTLOST) != 0);
                        this->GetDlgItem(IDC_RADIO_STATUS_PAPER_JAM)->EnableWindow(TRUE);
                        this->CheckDlgButton(IDC_RADIO_STATUS_PAPER_JAM, (aubC56DatPrinterStatus[0] & C56SR_PRINTER_PAPERJAM)     != 0);
                        this->GetDlgItem(IDC_RADIO_STATUS_PAPER_LOW)->EnableWindow(TRUE);
                        this->CheckDlgButton(IDC_RADIO_STATUS_PAPER_LOW, (aubC56DatPrinterStatus[0] & C56SR_PRINTER_PAPERLOW)     != 0);
                        this->GetDlgItem(IDC_RADIO_STATUS_PAPER_OUT)->EnableWindow(TRUE);
                        this->CheckDlgButton(IDC_RADIO_STATUS_PAPER_OUT, (aubC56DatPrinterStatus[0] & C56SR_PRINTER_OUTOFPAPER)   != 0);
                        this->GetDlgItem(IDC_RADIO_STATUS_POWER_ON)->EnableWindow(TRUE);
                        this->CheckDlgButton(IDC_RADIO_STATUS_POWER_ON, (aubC56DatPrinterStatus[0] & C56SR_PRINTER_POWERON)       != 0);
                     }
                  
                     //-- Update printer temperature
                     {  
                        char szBuff[20]; // Buffer

                        //-- Update dialog items
                        sprintf_s(szBuff, sizeof(szBuff),"%-3.3d", aubC56DatPrinterStatus[2]);
                        this->GetDlgItem(IDC_STATIC_STATUS_TEMP_C)->EnableWindow(TRUE);
                        this->GetDlgItem(IDC_STATIC_STATUS_TEMP_C)->SetWindowText(szBuff);
                        this->GetDlgItem(IDC_STATIC_STATUS_TEMP_C_TXT)->EnableWindow(TRUE);
                     
                        sprintf_s(szBuff, sizeof(szBuff),"%-3.3d", (signed int) ((aubC56DatPrinterStatus[2] * 1.8) + 32));
                        this->GetDlgItem(IDC_STATIC_STATUS_TEMP_F)->EnableWindow(TRUE);
                        this->GetDlgItem(IDC_STATIC_STATUS_TEMP_F)->SetWindowText(szBuff);
                        this->GetDlgItem(IDC_STATIC_STATUS_TEMP_F_TXT)->EnableWindow(TRUE);
                     }
                  
                     //-- Update printer hex code
                     {  
                        char szBuff[20]; // Buffer
                  
                        //-- Update dialog items
                        sprintf_s(szBuff, sizeof(szBuff),"[%2.2X][%2.2X][%2.2X][%2.2X]"
                                      , (WORD) (((BYTE *) &aubC56DatPrinterStatus)[0])
                                      , (WORD) (((BYTE *) &aubC56DatPrinterStatus)[1])
                                      , (WORD) (((BYTE *) &aubC56DatPrinterStatus)[2])
                                      , (WORD) (((BYTE *) &aubC56DatPrinterStatus)[3])
                                      );
                        this->GetDlgItem(IDC_STATIC_STATUS_HEX)->EnableWindow(TRUE);
                        this->GetDlgItem(IDC_STATIC_STATUS_HEX)->SetWindowText(szBuff);
                     }   
                  } 
                  else {
   
                     //--Invalide data in dialog  
                     {  
                        //-- Update dialog items
                        this->GetDlgItem(IDC_RADIO_STATUS_TEMP_ALERT)->EnableWindow(FALSE);
                        this->CheckDlgButton(IDC_RADIO_STATUS_TEMP_ALERT, FALSE);
                        this->GetDlgItem(IDC_RADIO_STATUS_LAST_PRINTOUT_LOST)->EnableWindow(FALSE);
                        this->CheckDlgButton(IDC_RADIO_STATUS_LAST_PRINTOUT_LOST, FALSE);
                        this->GetDlgItem(IDC_RADIO_STATUS_PAPER_JAM)->EnableWindow(FALSE);
                        this->CheckDlgButton(IDC_RADIO_STATUS_PAPER_JAM, FALSE);
                        this->GetDlgItem(IDC_RADIO_STATUS_PAPER_LOW)->EnableWindow(FALSE);
                        this->CheckDlgButton(IDC_RADIO_STATUS_PAPER_LOW, FALSE);
                        this->GetDlgItem(IDC_RADIO_STATUS_PAPER_OUT)->EnableWindow(FALSE);
                        this->CheckDlgButton(IDC_RADIO_STATUS_PAPER_OUT, FALSE);
                        this->GetDlgItem(IDC_RADIO_STATUS_POWER_ON)->EnableWindow(FALSE);
                        this->CheckDlgButton(IDC_RADIO_STATUS_POWER_ON, FALSE);
                     }

                     //-- Invalidate printer temperature
                     {  
                        //-- Update dialog items
                        this->GetDlgItem(IDC_STATIC_STATUS_TEMP_C)->EnableWindow(FALSE);
                        this->GetDlgItem(IDC_STATIC_STATUS_TEMP_C)->SetWindowText("-");
                        this->GetDlgItem(IDC_STATIC_STATUS_TEMP_C_TXT)->EnableWindow(FALSE);
                        this->GetDlgItem(IDC_STATIC_STATUS_TEMP_F)->EnableWindow(FALSE);
                        this->GetDlgItem(IDC_STATIC_STATUS_TEMP_F)->SetWindowText("-");
                        this->GetDlgItem(IDC_STATIC_STATUS_TEMP_F_TXT)->EnableWindow(FALSE);
                     }
    
                     //-- Invalidate printer hex code
                     {  
                        //-- Update dialog items
                        this->GetDlgItem(IDC_STATIC_STATUS_HEX)->EnableWindow(FALSE);
                        this->GetDlgItem(IDC_STATIC_STATUS_HEX)->SetWindowText("[??][??][??][??]");
                     }            
                  }
               } 
               else {

                  //-- Update printing status 
                  this->m_strError.Format("ERROR SENDING COMMAND FOR PRINTER STATUS \n");  
               }
                   
               //-- Close printer pipe-connection            
               C56Sdk_PioPrinterClose(hPrinterIo);   
            }  
         
            //-- Free allocated memory
            if (lpszPrinter) free(lpszPrinter); lpszPrinter = NULL; 
         }  
      }  
   } 
}

//******************************************************************************************
// METHOD:   CC56SdkDemo10DlgMain::OnTimer
// ABSTRACT: This member function is called in response to the WM_TIMER message.
//******************************************************************************************

VOID CC56SdkDemo10DlgMain::OnTimer
   (UINT_PTR nIdEvent)
{
   //
   if (nIdEvent == m_puiTimerUpdateStatus) {
  
      this->DoUpdateStatus();
   }
   else 
   if (nIdEvent == m_puiTimerPrintout) {

      this->DoPrintout(TRUE);
   }
   else {
      //-- Other dialog timer
      CDialog::OnTimer(nIdEvent);

   }
}

//******************************************************************************************
// METHOD:   CC56SdkDemo10DlgMain::DoPrintout
// ABSTRACT: Handler for printout button being pressed
//******************************************************************************************

void CC56SdkDemo10DlgMain::DoPrintout
   (BOOL fAuto)
{
   //-- Reset printer status / format
   if (fAuto) {
      this->m_strStatus.Format("-");
      this->m_strError.Format("-");
   }

   //-- Check if there are still tickets to print
   if (!fAuto || (this->m_dwNrOfTickets > 0)) {
    
      //-- Single ticket or auto
      if (fAuto) {

         DWORD dwTimeToGo;                                  // Estimated time to go in sec

         //-- Determine time to go in sec
         dwTimeToGo = (this->m_dwNrOfTickets * max(1, m_dwIntervalMs)) / 1000;

         //-- Update printing status 
         this->m_strStatus.Format("PRINTING TICKET %lu\n%lu TICKETS TO PRINT\nESTIMATED TIME TO GO %2.2lu:%2.2lu:%2.2lu", this->m_dwTicketNr+1, this->m_dwNrOfTickets-1, dwTimeToGo / 3600, (dwTimeToGo % 3600) / 60, dwTimeToGo % 60);  
      }
      else {

         //-- Update printing status 
         this->m_strStatus.Format("\nPRINTING SINGLE TICKET\n");  
      }

      HANDLE hPrinterIo; // Handle to printer I/O
      
      //-- Open connection to printer
      if ((hPrinterIo = C56Sdk_PioPrinterOpen((LPSTR)(LPCSTR) this->m_strPrinter,5000)) != NULL) {
         
         //-- Set the print quality to High
         {
            unsigned char PrintQuality[] = {0x1D, 0x45 , 0x01};
            C56SDK_PIOPRINTERWRITE_BYTEARRAY(hPrinterIo,PrintQuality, 1000);
         }

         //-- Set line spacing
         {
            unsigned char aubLineSpacing[] = {0x1B, 0x41 , 0x00};
            C56SDK_PIOPRINTERWRITE_BYTEARRAY(hPrinterIo, aubLineSpacing, 1000);
         }
   
         //-- Set the print mode,8x16 charakter typeface,no magnification,no double width & no double height
         {
            unsigned char aubPrintMode[] = {0x1B, 0x21 , 0x00};
            C56SDK_PIOPRINTERWRITE_BYTEARRAY(hPrinterIo, aubPrintMode, 1000);
         }
         
         //-- Add text to print
         C56SDK_PIOPRINTERWRITE_STRING(hPrinterIo, "=======================================================\n", 1000);
         C56SDK_PIOPRINTERWRITE_STRING(hPrinterIo, "                   START OF TICKET\n"                     , 1000);
         C56SDK_PIOPRINTERWRITE_STRING(hPrinterIo, "-------------------------------------------------------\n", 1000);
         C56SDK_PIOPRINTERWRITE_STRING(hPrinterIo, "                   C56 SDK DEMO 10\n"                     , 1000);
         C56SDK_PIOPRINTERWRITE_STRING(hPrinterIo, "-------------------------------------------------------\n", 1000);
         
         //-- Add ticket info
         {
            CString    oString;       // Data string
            CTime      oTime;         // Time object 
                
            //-- Get current time and date  
            oTime = CTime::GetCurrentTime();
            
            //-- Write date of print
            {
               C56SDK_PIOPRINTERWRITE_STRING(hPrinterIo, "    Date of printout   : ", 1000);
               oString = oTime.Format( "%B %d, %Y");
               C56SDK_PIOPRINTERWRITE_STRING(hPrinterIo, oString.GetBuffer(), 1000); oString.ReleaseBuffer();
               C56SDK_PIOPRINTERWRITE_STRING(hPrinterIo, "\n", 1000);
            }

            //-- Write time of print
            {
               C56SDK_PIOPRINTERWRITE_STRING(hPrinterIo, "    Time of printout   : ", 1000);
               oString = oTime.Format( "%H:%M:%S");
               C56SDK_PIOPRINTERWRITE_STRING(hPrinterIo, oString.GetBuffer(), 1000); oString.ReleaseBuffer();
               C56SDK_PIOPRINTERWRITE_STRING(hPrinterIo, "\n", 1000);
            }

            //-- Check for auto printout and print ticket number if so
            if (fAuto) {

               //-- Write ticket number 
               {
                  C56SDK_PIOPRINTERWRITE_STRING(hPrinterIo, "    Ticket number      : ", 1000);
                  oString.Format("%d", this->m_dwTicketNr);
                  C56SDK_PIOPRINTERWRITE_STRING(hPrinterIo, oString.GetBuffer(), 1000); oString.ReleaseBuffer();
                  C56SDK_PIOPRINTERWRITE_STRING(hPrinterIo, "\n", 1000);
               }

               //-- Write print interval
               {
                  C56SDK_PIOPRINTERWRITE_STRING(hPrinterIo, "    Print interval     : ", 1000);
                  oString.Format("%d MS", this->m_dwIntervalMs);
                  C56SDK_PIOPRINTERWRITE_STRING(hPrinterIo, oString.GetBuffer(), 1000); oString.ReleaseBuffer();
                  C56SDK_PIOPRINTERWRITE_STRING(hPrinterIo, "\n", 1000);
               }
            }
            
            //-- Add text to print
            C56SDK_PIOPRINTERWRITE_STRING(hPrinterIo, "=======================================================\n", 1000);
         }
          
         //-- Ticket data
         {
            //-- Set the print mode,8x16 charakter typeface,no magnification, double width & double height
            {
               unsigned char aubPrintMode[] = {0x1B, 0x21 , 0x30};
               C56SDK_PIOPRINTERWRITE_BYTEARRAY(hPrinterIo, aubPrintMode, 1000);
            }
         
            //-- Set the line feed 
            {
               unsigned char aubLineFeed[] = {0x1B, 0x4A , 0x20};
               C56SDK_PIOPRINTERWRITE_BYTEARRAY(hPrinterIo, aubLineFeed, 1000);
            }
         
            //-- Set tab stops to ... 
            {  
               unsigned char aubTabs[] = {0x1b, 0x44, 0x04, 0x00};
               C56SDK_PIOPRINTERWRITE_BYTEARRAY(hPrinterIo, aubTabs, 1000);
            }
            
            //-- Reverse printing
            {   
               //-- Enable reverse printing
               {
                  unsigned char aubRevPrint[] = {0x1B, 0x1E };
                  C56SDK_PIOPRINTERWRITE_BYTEARRAY(hPrinterIo, aubRevPrint, 1000); 
               }

               //-- Add text to print
               {
                  C56SDK_PIOPRINTERWRITE_STRING(hPrinterIo, "\t                      \n", 1000);      
                  C56SDK_PIOPRINTERWRITE_STRING(hPrinterIo, "\t   H E N G S T L E R  \n", 1000);
                  C56SDK_PIOPRINTERWRITE_STRING(hPrinterIo, "\t                      \n", 1000);
               }

               //-- Disable reverse printing
               {
                  unsigned char aubRevPrint2[] = {0x1B, 0x1F };
                  C56SDK_PIOPRINTERWRITE_BYTEARRAY(hPrinterIo, aubRevPrint2, 1000);
               }
            }

            //-- Add the line feed
            {
               unsigned char aubLineFeed[] = {0x1B, 0x4A , 0x20};
                C56SDK_PIOPRINTERWRITE_BYTEARRAY(hPrinterIo, aubLineFeed, 1000);      
            }
         
            //-- Set info on hengstler
            C56SDK_PIOPRINTERWRITE_STRING(hPrinterIo, "\tUhlandstrasse 49\n\t78554 Aldingen\n\tGermany\n\tTel. +49-(0)7424-890\n\tinfo@hengstler.com\n\twww.hengstler.com\n", 1000);

            //-- Set the line feed
            {
               unsigned char aubLineFeed[] = {0x1B, 0x4A , 0x20};
               C56SDK_PIOPRINTERWRITE_BYTEARRAY(hPrinterIo, aubLineFeed, 1000);   
            }
         
            //-- Add date/time
            {
               CString    oString;       // Data string
               CTime      oTime;         // Time object 

               //-- Get current date and time
               oTime = CTime::GetCurrentTime();
            
               //-- Write date
               {
                  oString.Format("   %s", (LPSTR) (LPCSTR) oTime.Format( "%y/%m/%d"));
                  C56SDK_PIOPRINTERWRITE_STRING(hPrinterIo, oString.GetBuffer(), 1000); oString.ReleaseBuffer();
               }

               //-- Write time
               {
                  oString.Format("       %s", (LPSTR) (LPCSTR) oTime.Format("%H:%M"));
                  C56SDK_PIOPRINTERWRITE_STRING(hPrinterIo, oString.GetBuffer(), 1000); oString.ReleaseBuffer();
               }
            }

            //-- Set the line feed
            {
               unsigned char aubLineFeed[] = {0x1B, 0x4A , 0x20};
               C56SDK_PIOPRINTERWRITE_BYTEARRAY(hPrinterIo, aubLineFeed, 1000);
            }
            
            //-- Add ticket data
            C56SDK_PIOPRINTERWRITE_STRING(hPrinterIo, "\t001 996278984 (1)\n\t 2,00 * 4,00 99\n\t 500 g./m2      8,00\n\t001 996278982 (1)\n\t 1,00 * 4,00 99\n\t 500 g./m2     12,00\n\t001 996278984 (1)\n\t 2,00 * 4,00 99\n\t 500 g./m2      8,00\n\t            --------\n\tTotal EUR      38,00\n", 1000);

            //-- Add the line feed
            {
               unsigned char aubLineFeed[] = {0x1B, 0x4A , 0x20};
               C56SDK_PIOPRINTERWRITE_BYTEARRAY(hPrinterIo, aubLineFeed, 1000);
            }
      
            //-- Add barcode 
            {
               //-- Add barcode width
               {
                  unsigned char aubBarCodeW[] = {0x1D, 0x65, 0x02, 0x06};
                  C56SDK_PIOPRINTERWRITE_BYTEARRAY(hPrinterIo, aubBarCodeW, 1000);
               }

               //-- Add barcode height
               {
                  unsigned char aubBarCodeH[] = {0x1D, 0x68, 0x30};                              
                  C56SDK_PIOPRINTERWRITE_BYTEARRAY(hPrinterIo, aubBarCodeH, 1000);
               }

               //-- Add barcode magnification
               {
                  unsigned char aubBarCodeM[] = {0x1D, 0x77, 0x01};       
                  C56SDK_PIOPRINTERWRITE_BYTEARRAY(hPrinterIo, aubBarCodeM, 1000);
               }

               //-- Add single tab
               {
                  unsigned char aubTab[] = "\t"; 
                  C56SDK_PIOPRINTERWRITE_BYTEARRAY(hPrinterIo, aubTab, 1000);
               }

               //-- Add sample barcode
               {
                  unsigned char aubBarCode[] = {0x1D, 0x6B,  0x69,  0x08,  0x30,  0x31,  0x32,  0x33, 0x34, 0x35, 0x36, 0x37};      
                  C56SDK_PIOPRINTERWRITE_BYTEARRAY(hPrinterIo, aubBarCode, 1000);
               }

               //-- Add new lines
               {
                  unsigned char aubNewLine[] = "\n\n";
                  C56SDK_PIOPRINTERWRITE_BYTEARRAY(hPrinterIo, aubNewLine, 1000);
               }
            }
         }
         
         //-- Add ticket footer
         {         
            //-- Set the print mode,8x16 charakter typeface,no magnification,no double width & no double height
            {
               unsigned char aubPrintMode[] = {0x1B, 0x21 , 0x00};
               C56SDK_PIOPRINTERWRITE_BYTEARRAY(hPrinterIo, aubPrintMode, 1000);
            }
            
            //-- Add text to print
            {
               C56SDK_PIOPRINTERWRITE_STRING(hPrinterIo, "=======================================================\n", 1000);
               C56SDK_PIOPRINTERWRITE_STRING(hPrinterIo, "                   END OF TICKET\n", 1000);
               C56SDK_PIOPRINTERWRITE_STRING(hPrinterIo, "=======================================================\n", 1000);
            }
         }

         //-- Close connection to printer
         C56Sdk_PioPrinterClose(hPrinterIo);
            
         //-- Increase ticket number if auto 
         if (fAuto) 
            this->m_dwTicketNr++;

          //-- Decrease nr of tickes if auto
         if (fAuto) 
            this->m_dwNrOfTickets--;
      }
      else {
 
         //-- Update printing status 
         this->m_strError.Format("STARTING DOCUMENT FAILED");   
      }
   }
   else {

      //-- Update printing status 
      this->m_strStatus.Format("\nTICKET PRINTOUT COMPLETED");  
   }

   //-- Update dialog items
   this->SetDlgItems();
}
